
// Include Standard files
#include <inarm.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"

extern void asm_SWIRQ0_Handler_Entry(void);
extern void asm_ST0_Handler_Entry(void);
extern void asm_FIQ_Handlesr(void);

CSP_GIC_T * const GIC     = ((CSP_GIC_T *)   GIC_BASE_ADDRESS);
CSP_ST_T * const ST0     = ((CSP_ST_T *)   ST0_BASE_ADDRESS);


extern CSP_PWM4C_T * const PWM4C;

unsigned char ch;

unsigned long counter=0;

void TimerBeat(void);


__arm __irq void c_st0_interrupt(void) {

  //
  counter++;

  // Call this function
  TimerBeat();

  // Clear Timer Interupt
  ST0->CSR = 1;

  // End of interrupt
  GIC->EOICR = 0;

}


// AMC -----------------------------------------------------------------------

CSP_AMC_T * const AMC = (CSP_AMC_T *)AMC_BASE_ADDRESS;

void InitAMC(void) {

  // Init AMC
  AMC->MCR = ALE_2;

  AMC->CSR[0] = (BA & 0x40000000) | // offset 2MB
                CSEN_ENABLE       | // CS Enable
                BAT_SELECT        | // Byte access enable
                TDF_7             | // Data Float Output Time
                PAGES_4           | // Page size 4MB
                WSE_ENABLE        | // Wait statet enable
                NWS_8             | // 8 wait states
                DBW_16;             // Data Bus Width 16 bits

  AMC->CSR[1] = (BA & 0x40400000) | // offset 2MB
                CSEN_ENABLE       | // CS Enable
                BAT_SELECT        | // Byte access enable
                TDF_0             | // Data Float Output Time
                PAGES_4           | // Page size 4MB
                WSE_ENABLE        | // Wait statet enable
                NWS_3             | // 8 wait states
                DBW_16;             // Data Bus Width 16 bits

  AMC->CSR[2] = (BA & 0x40800000) | // offset 2MB
                CSEN_ENABLE       | // CS Enable
                BAT_SELECT        | // Byte access enable
                TDF_0             | // Data Float Output Time
                PAGES_4           | // Page size 4MB
                WSE_ENABLE        | // Wait statet enable
                NWS_3             | // 8 wait states
                DBW_16;             // Data Bus Width 16 bits

}
// End AMC --------------------------------------------------------------------


// Timer ----------------------------------------------------------------------
void InitST0(void) {

  #define CORECLK_FREQUENCY 30000000

  // Itit Timer ST0
  // Set up timer
  ST0->ECR  = ST;         // Enable simple timer clock
  ST0->CR   = CHDIS0;     // Disable simple timer channel 0

  ST0->CHANNEL[0].PR = 0x00000020 | (((CORECLK_FREQUENCY / 2000000) - 1) << 8); // Setup prescaler for 1Mhz timer frequency
  ST0->CHANNEL[0].CT = 999;         // Generate a 1Khz timer interrupt

  ST0->IER  |= 1 << 0;   // Generate interrupt when timer reaches zero
  ST0->CR   = CHEN0;     // Enable simple timer channel 0



  // Interrupt -----------------------------------------------------------
  // Put LDR PC, [PC, #-0xF20] instruction into IRQ vector in order to use
  // hardware interrupt vectoring.
  *(volatile unsigned int *)0x00000018 = 0xE51FFF20;

  // Interrupt IRQ
  // Disable & CLear All Interrupts
  CSP_GIC_SET_IDCR(GIC, 0xFFFFFFFF);
  CSP_GIC_SET_ICCR(GIC, 0xFFFFFFFF);
  // Disable the IRQ1 interrupt on the interrupt controller
  GIC->IDCR =  IBIT_ST0;
  // Save the interrupt handler routine pointer and the interrupt priority
  GIC->SVR[INT_22] = (unsigned int)c_st0_interrupt;
  // Store the Source Mode Register
  GIC->SMR[INT_22] = ( LOW_LEVEL_SENSITIVE | PRIOR_7 );
  // Clear the interrupt on the interrupt controller
  GIC->ICCR = IBIT_ST0 ;
  // Ebnable IT
  GIC->IECR = IBIT_ST0 ;
  // Software interrupt
  GIC->ISCR = IBIT_ST0 ;

  __enable_interrupt();

}

// End Timer ------------------------------------------------------------------

// Uip ------------------------------------------------------------------------


#define   UIP_ARP_TIMER   10000
#define   UIP_TX_TIMER    500
#define   LED_TIMER       500

#define _BUF ((struct uip_eth_hdr *)&uip_buf[0])

int TX_EventPending;	        // the DM9000 hardware receive event
int ARP_EventPending;	        // trigger the arp timer event
unsigned long half_Sec;
unsigned long ten_Secs;


void TimerBeat(void)
{
  half_Sec--;                           // Decrement half sec counter
  if (half_Sec == 0)                    // Count 1ms intervals for half a second
  {			
    half_Sec = UIP_TX_TIMER;            // and then set the receive poll flag
    TX_EventPending = TRUE;             // the DM9000 hardware receive.
  }

  ten_Secs--;
  if (ten_Secs == 0)                    // Count 1ms intervals for ten seconds
  {
    ten_Secs = UIP_ARP_TIMER;           // and then set the event required to
    ARP_EventPending = TRUE;            // trigger the arp timer if necessary
  }

}
// End Uip --------------------------------------------------------------------

//*----------------------------------------------------------------------------
//* Function Name       : main
//* Object              : Main application. It swings LEDs.
//*----------------------------------------------------------------------------
int main (void)
{

  unsigned int i;

  InitAMC();

  UARTInitialize(9600);

  InitST0();

  uip_init();               // Initialise the uIP TCP/IP stack.
  httpd_init();             // Initialise the webserver app.

  if (InitDM9000())         // Initialise the device driver.
  {
   UARTString("Cannot Identify DM9000 device");
  }

  TX_EventPending = FALSE;	// False to poll the DM9000 receive hardware
  ARP_EventPending = FALSE;	// clear the arp timer event flag

  // loop forever
  while(1) {


    // Loops here until either a packet has been received or
    // TX_EventPending (half a sec) to scan for anything to send or
    // ARP_EventPending (10 secs) to send an ARP packet		
    uip_len = DM9000_receive();

    // If uip_len is 0, no packet has been received
    if (uip_len == 0)
    {

      // Test for anything to be sent to 'net
      if (TX_EventPending)
      {
        TX_EventPending = FALSE;  // First clear flag if set

        // UIP_CONNS - nominally 10 - is the maximum simultaneous
        // connections allowed. Scan through all 10
        for (i = 0; i < UIP_CONNS; i++)
        {
          uip_periodic(i);
          // If uip_periodic(i) finds that data that needs to be
          // transmitted to the network, the global variable uip_len
          // will be set to a value > 0.
          if (uip_len > 0)
          {
            // The uip_arp_out() function should be called when an IP packet should be sent out on the
            // Ethernet. This function creates an Ethernet header before the IP header in the uip_buf buffer.
            // The Ethernet header will have the correct Ethernet MAC destination address filled in if an
            // ARP table entry for the destination IP address (or the IP address of the default router)
            // is present. If no such table entry is found, the IP packet is overwritten with an ARP request
            // uip_len variable holds the length of the Ethernet frame that should be transmitted.
            uip_arp_out();
            DM9000_transmit();
          }
        }

#if UIP_UDP
        for (i = 0; i < UIP_UDP_CONNS; i++)
        {
          uip_udp_periodic(i);
          // If the above function invocation resulted in data that
          // should be sent out on the network, the global variable
          // uip_len is set to a value > 0.
          if (uip_len > 0)
          {
            uip_arp_out();
            DM9000_transmit();
          }
        }
#endif /* UIP_UDP */
      }

      // Call the ARP timer function every 10 seconds. Flush dead entries
      if (ARP_EventPending)
      {
        ARP_EventPending = FALSE;
        uip_arp_timer();
      }
    }
    // Packet Received (uip_len != 0) Process incoming
    else
    {
      if (_BUF->type == htons(UIP_ETHTYPE_IP))
      {
        // Received an IP packet
        uip_arp_ipin();
        uip_input();
        // If the above function invocation resulted in data that
        // should be sent out on the network, the global variable
        // uip_len is set to a value > 0.
        if (uip_len > 0)
        {
          uip_arp_out();
          DM9000_transmit();
        }
      }
      else
      {
        if (_BUF->type == htons(UIP_ETHTYPE_ARP))
        {
          // Received an ARP packet
          uip_arp_arpin();
          // If the above function invocation resulted in data that
          // should be sent out on the network, the global variable
          // uip_len is set to a value > 0.
          if (uip_len > 0)
          {	
            DM9000_transmit();
          }
        }
      }
    }
  }	// end of 'while (1)'	
}
